local dark_addon = _G["dark_interface"]

unlocker = { }
local myOM = {}

function unlocker.canloot(guid)
    return UnitCanBeLooted(myOM[guid])
end

function unlocker.distance(guid)
    return GetDistanceBetweenObjects('player', myOM[guid])
end

local function face_direction(obj)
	if not UnitExists("target") or UnitIsDeadOrGhost("target") then
		return
	end
	local nx, ny  -- a point north of the player
	local px, py  -- the player
	local mx, my  -- the monster
	-- get location of player and mob
    px, py = ObjectPosition("player")
	mx, my = ObjectPosition("target")
	-- pick some point directly north of player
	nx = px + 20
	ny = py

	-- A is the distance between n and m
	-- B is the distance between p and m
	-- C is the distance between p and n
	local a = GetDistanceBetweenPositions(nx, ny, 0, mx, my, 0)
	local b = GetDistanceBetweenPositions(px, py, 0, mx, my, 0)
	local c = GetDistanceBetweenPositions(px, py, 0, nx, ny, 0)
	-- cosine law ftw
	local numerator = (b ^ 2) + (c ^ 2) - (a ^ 2)
	local denominator = 2 * b * c
	if denominator == 0 then
		return
	end
	local angle = math.acos(numerator / denominator)
	if my < py then
		angle = 2 * math.pi - angle
	end
	local minangle = angle - (math.pi / 4)
	local maxangle = angle + (math.pi / 4)
	local playerfacing = 0
	playerfacing = ObjectFacing("player")
	if playerfacing < minangle or playerfacing > maxangle then
		FaceDirection(angle, true)
	end
end

function unlocker.face(guid)
    local obj = myOM[guid]
    FaceDirection(obj, true)
    --face_direction(obj)
end


function unlocker.interact(guid)
    ObjectInteract(myOM[guid])
end

function unlocker.position(guid)
    -- for some reason, returning out of unit.lua's __index metamethod only
    -- returns 1 value.  that's why this code packs the position into an array
    -- i don't like it either...
    return { ObjectPosition(myOM[guid]) }
end

function unlocker.target(guid)
    TargetUnit(myOM[guid])
end

--function unlocker.goto(x, y, z)
--    GoTo(x, y, z)
--end

---[[
local MeshIsInit = false
local MeshOnce = false
local MeshPath = nil
local MeshNextPoint = 0
local MeshX, MeshY, MeshZ
local function reset_nav()
    MeshPath = nil
    MeshNextPoint = 0
    MeshX = 0
    MeshY = 0
    MeshZ = 0
    --print('clear nav path')
end
function unlocker.goto(x, y, z)
    --print('unlocker.goto(', x, ',', y, ',', z, '): called...')
    if not MeshOnce then
        MeshOnce = true
        InitializeNavigation(function(result)
            if result then MeshIsInit = true
            else dark_addon.error("Failed to init EWT NavMesh") end
        end)
    end

    -- if called with a different destination, reset everyting
    if MeshX ~= x and MeshY ~= y and MeshZ ~= z then
        reset_nav()
    end

    local px, py, pz = ObjectPosition("Player");
    if GetDistanceBetweenPositions(px, py, pz, x, y, z) < 5 then
        --print('unlocker.goto(): you have arrived...')
        MoveForwardStop()
        reset_nav()
        return
    end

    if not MeshIsInit then
        print('unlocker.goto(): waiting for navmesh init...')
        return
    end

    if MeshPath == nil then
        local totalDist
        MeshPath, totalDist = CalculatePath(GetMapId(), px, py, pz, x, y, z)
        MeshNextPoint = 1
        MeshX = x
        MeshY = y
        MeshZ = z
        --print('unlocker.goto(): got path.  target is ', totalDist, ' yards away')
    end

    --print('unlocker.goto(): there are ', #MeshPath, ' hops...')

    local hx = MeshPath[MeshNextPoint][1]
    local hy = MeshPath[MeshNextPoint][2]
    local hz = MeshPath[MeshNextPoint][3]
    if GetDistanceBetweenPositions(px, py, pz, hx, hy, hz) > 1 then
        --print('calling MoveTo(', hx, ',', hy, ',', hz, ')...')
        MoveTo(hx, hy, hz)
        return
    else
        --print('unlocker.goto(): next hop...')
        MeshNextPoint = MeshNextPoint + 1
        return
    end
end
--]]

function unlocker.stop()
    reset_nav()
    while GetUnitSpeed('player') > 0 do
        local px, py, pz = ObjectPosition("player")
        --print('calling stop MoveTo(', px, ',', py, ',', pz, ')...')
        MoveTo(px, py, pz)
        MoveForwardStop()
        coroutine.yield()
    end
end

function unlocker.guid_from_name(name)
    for guid, obj in pairs(myOM) do
        if ObjectName(obj) == name then
            return guid
        end
    end
end

function unlocker.distance_between_positions(x1, y1, z1, x2, y2, z2)
    return GetDistanceBetweenPositions(x1, y1, z1, x2, y2, z2)
end

-----------------
-----------------
-- Object Manager
-----------------
-----------------

local initOM = true
local myFrame = CreateFrame("Frame")
myFrame:SetScript("OnUpdate", function ()
    -- The first frame call to GetObjectCount(true) should have total == #added and 'added' will contain all objects.
    -- The next GetObjectCount(true) calls should have only the updated objects.
    local total, updated, added, removed = GetObjectCount(true, "dark_addon")
    if initOM then
        initOM = false
        for i = 1,total do
            local thisUnit = GetObjectWithIndex(i)  -- thisUnit contains the '0x' string representing the object address 
            -- Do something with the unit
            if ObjectIsUnit(thisUnit) then
                print('initial om init added ', ObjectName(thisUnit))
                myOM[ObjectGUID(thisUnit)] = thisUnit
            end
        end
    end
    for k,v in pairs(added) do
        -- k - Number = Array index = Don't confuse this with the Object Index used by GetObjectWithIndex. It's not the same thing.
        -- v - String = Object address
        if ObjectIsUnit(v) then
            --print('delta added ' .. ObjectName(v))
            myOM[ObjectGUID(v)] = v
        end
    end
    for k,v in pairs(removed) do
        --print('Removed ' .. v)
        myOM[v] = nil
    end
end);
